% Filtre Kalman Etendu pour la fusee qui decolle avec capteur pression
% Introduction a la robotique mobile GLO-4001/GLO-7021
% (c) Philippe Giguere 2012
clearvars
clf
myFontSize = 14;

% Variables inconnues dcrivant le systme.
Sa         = 0.5; % Bruit sur l'acceleration
Spression  = 0.05;  % Bruit sur le capteur de pression
xVrai = [0;0;0];
C = 1.244e-4;  % constante pour le capteur de pression
P0 = 101;   % Pression au sol


% Variables connues
P = zeros(3,3);
X = xVrai;

% Intervalle de temps
T = 0.1;  

% Matrices
F = [1 T 0; 0 1 0; 0 0 0];
G = [0.5*T^2; T; 1];
Cv = (Sa)^2;
Cw = Spression^2;
  
a  = 5;   % Acceleration constante, en m/s^2

for iStep = 1:60
    % Simulation du systeme
    Accel = a + Sa*randn();
    xVrai(1) = xVrai(1) + xVrai(2)*T + 0.5*T^2*Accel;
    xVrai(2) = xVrai(2) + T*Accel;
    xVrai(3) = Accel;
    
    z  = P0*exp(-C*xVrai(1)) + Spression*randn; % mesure de pression
    
    % Et maintenant le filtre de Kalman
    U = a;
    
    % ====== Propagation =========
    X = F*X + G*U;
    P = F*P*F' + G*Cv*G';

    % Jacobienne pour la mesure
    L = [-C*P0*exp(-C*X(1)) 0 0];
    %L = [0 0 0];

    
    % ======== Mise-a-jour ========
    K = P*L'/(L*P*L'+Cw);
    zhat = P0*exp(-C*X(1));
    X = X + K*(z-zhat);
    P = (eye(size(P,2))-K*L)*P;
    
    
    Ax(iStep) = xVrai(1);
    AEx(iStep) = X(1);
    AZ(iStep) = z;
    AEFromZ(iStep) = -log(z/P0)/C;
    ErreurEx(iStep) = X(1)-xVrai(1);
    ErreurZ(iStep) = AEFromZ(iStep)-xVrai(1);
    ATime(iStep) = iStep*T;
end

subplot(3,1,[1 2]);
hl(3) = plot(ATime,AEx,'ko'); %,'MarkerFaceColor','g');
hold on;
hl(2) = plot(ATime,AEFromZ,'kx'); % Le facteur 1/3 est pour inverser la fonction de capteur.
hl(1) = plot(ATime,Ax,'k-');
xlabel('Temps (s)','FontSize',myFontSize);
ylabel('Altitude (m)','FontSize',myFontSize);
legend(hl,{'Altitude reelle' 'Altitude pression (fonction inverse)' 'Altitude Kalman'},'Location','NorthWest');
set(gca,'FontSize',myFontSize-2);

subplot(3,2,5)
hist(ErreurZ);
title('Distribution des erreurs avec le capteur pression');
xlabel('Erreur (m)');
c = axis();

subplot(3,2,6)
hist(ErreurEx);
title('Distribution des erreurs avec le filtre de Kalman');
xlim(c(1:2));
xlabel('Erreur (m)');

print -deps KalmanEtenduFusee.eps

