% Filtre Kalman pour un pont roulant X-Y
% Introduction a la robotique mobile GLO-4001/GLO-7021
% (c) Philippe Giguere 2012
clearvars
clf
myFontSize = 14;

% Variables inconnues dcrivant le systme.
SlaserX  = 0.2;  % Bruit sur le laser X
SlaserY  = 0.2;  % Bruit sur le laser Y
SVeloY   = 0.1; % Bruit sur la vitesse Y
SmoteurX = 0.02;
SmoteurY = 0.04;

xVrai = [0;0;0;0];

% Variables connues
P = zeros(4,4);
X = xVrai;

% Intervalle de temps
T = 0.1;  

% Matrices
F = [1 0 0 0; 0 0 0 0; 0 0 1 0; 0 0 0 0];
G = [5*T 0; 5 0; 0 6*T; 0 6];
L = [2 0 0 0; 0 0 3 0; 0 0 0 4];
Cv = [SmoteurX^2 0; 0 SmoteurY^2];
Cw = [SlaserX^2 0 0; 0 SlaserY^2 0; 0 0 SVeloY^2];
  
for iStep = 1:40
    % Simulation du systeme
    U = [0.2; 0.1];

    Vx = 5*(U(1) + SmoteurX*randn());
    Vy = 6*(U(2) + SmoteurY*randn());
    xVrai(1) = xVrai(1) + Vx*T;
    xVrai(2) = Vx;
    xVrai(3) = xVrai(3) + Vy*T;
    xVrai(4) = Vy;
    z(1,1)  = 2*xVrai(1) + SlaserX*randn;
    z(2,1)  = 3*xVrai(3) + SlaserY*randn;
    z(3,1)  = 4*xVrai(4) + SVeloY*randn;
    
    % Et maintenant le filtre de Kalman
    
    
    % ====== Propagation =========
    X = F*X + G*U;
    P = F*P*F' + G*Cv*G';
    
    % ======== Mise-a-jour ========
    K = P*L'/(L*P*L'+Cw);
    X = X + K*(z-L*X);
    P = (eye(size(P,2))-K*L)*P;
    
    
    Ax(iStep) = xVrai(1);
    Ay(iStep) = xVrai(3);
    AEx(iStep) = X(1);
    AEy(iStep) = X(3);
    AZX(iStep) = z(1)/2; % Le facteur 1/2 est pour inverser la fonction de capteur.
    AZY(iStep) = z(2)/3; % Le facteur 1/3 est pour inverser la fonction de capteur.
    ErreurEx(iStep) = X(1)-xVrai(1);
    ErreurEy(iStep) = X(3)-xVrai(3);
    ErreurLaserX(iStep) = AZX(iStep)-xVrai(1); 
    ErreurLaserY(iStep) = AZY(iStep)-xVrai(3); 
    ATime(iStep) = iStep*T;
end

subplot(3,1,[1 2]);
hl(3) = plot(AEx,AEy,'ko'); %,'MarkerFaceColor','g');
hold on;
hl(2) = plot(AZX,AZY,'kx');
hl(1) = plot(Ax,Ay,'k-');
xlabel('Position x (m)','FontSize',myFontSize);
ylabel('Position y (m)','FontSize',myFontSize);
legend(hl,{'Position reelle' 'Position laser (fonction inverse)' 'Position Kalman'},'Location','NorthWest');
set(gca,'FontSize',myFontSize-2);
axis equal

mylim = [-0.4 0.4];
subplot(3,4,9)
hist(ErreurLaserX);
title({'Distribution des' 'erreurs laser X'});
xlabel('Erreur (m)');
xlim(mylim);

subplot(3,4,10)
hist(ErreurLaserY);
title({'Distribution des' 'erreurs laser Y'});
xlabel('Erreur (m)');
xlim(mylim);

subplot(3,4,11)
hist(ErreurEx);
title({'Distribution des' 'erreurs Kalman X'});
xlabel('Erreur (m)');
xlim(mylim);


subplot(3,4,12)
hist(ErreurEy);
title({'Distribution des' 'erreurs Kalman Y'});
xlabel('Erreur (m)');
xlim(mylim);

print -deps KalmanPontRoulant.eps



